let DMC = require('dmc.js');
var AsciiTable = require('ascii-table');
let config = require("./config.json")

let account = "";
let private_key = "";

console.notice("------- 多签转账脚本 -------\n");
while (true) {
    account = console.readLine("请输入用户名:\n");
    if (!account) continue;
    private_key = console.getpass("请输入私钥:\n")
    if (!private_key) continue;
    break;
}

let httpEndpoint = config.httpEndpoint;
let proposalName = config.propose.proposalName;
let from = config.propose.from;
let to = config.propose.to;
let quantity = config.propose.quantity;
let memo = config.propose.memo;
//初始化客户端
let dmc = DMC({
    chainId: config.chainId,
    keyProvider: private_key,
    httpEndpoint: httpEndpoint,
    logger: {
        log: null,
        error: null
    }
});

console.notice("--------- connect to " + httpEndpoint + "--------\n--------- welcome " + account + " ---------");
let requested = []
config.accounts.forEach(e => {
    requested.push({
        actor: e,
        permission: "active"
    })
})

function choice() {
    console.log(
        `
1. propose\n
2. list approvals\n
3. exec propose\n
4. cancel propose\n
`
    )
    let choose = Number(console.readLine("choice:"));
    switch (choose) {
        case 1:
            let actions = [{
                'account': 'dmc.token',
                'name': 'extransfer',
                'authorization': [{
                    "actor": from,
                    "permission": "active"
                }],
                'data': {
                    "from": from,
                    "to": to,
                    "quantity": quantity,
                    "memo": memo
                }
            }]
            let serializeActions = dmc.serializeActions_sync(actions);
            dmc.transact_sync({
                actions: [{
                    account: 'dmc.msig',
                    name: 'propose',
                    authorization: [
                        {
                            actor: account,
                            permission: 'active'
                        },
                    ],
                    data: {
                        "proposer": account,
                        "proposal_name": proposalName,
                        "requested": requested,
                        "trx": {
                            "expiration": "2024-01-01T00:30",
                            "ref_block_num": 0,
                            "ref_block_prefix": 0,
                            "max_net_usage_words": 0,
                            "max_cpu_usage_ms": 0,
                            "delay_sec": 0,
                            "context_free_actions": [],
                            "actions": serializeActions,
                            "transaction_extensions": []
                        }
                    }
                }]
            }, {
                blocksBehind: 3,
                expireSeconds: 30,
            });
            break;
        case 2:
            showApprovals(account)
            break;
        case 3:
            dmc.transact_sync({
				actions: [{
					account: 'dmc.msig',
					name: 'exec',
					authorization: [
						{
							actor: account,
							permission: 'active'
						},
					],
					data: {
						"proposer": account,
						"proposal_name": proposalName,
						"executer": account
					}
				}]
			}, {
				blocksBehind: 3,
				expireSeconds: 30,
			});
            break;
        case 4:
            dmc.transact_sync({
				actions: [{
					account: 'dmc.msig',
					name: 'cancel',
					authorization: [
						{
							actor: account,
							permission: 'active'
						},
					],
					data: {
						"proposer": account,
						"proposal_name": proposalName,
						"canceler": account
					}
				}]
			}, {
				blocksBehind: 3,
				expireSeconds: 30,
			});
            break;
    }
    choice();
}
choice()
function showApprovals(pname) {
    console.notice("-------show " + pname + "`s approvals-------")
    var a = dmc.rpc.get_table_rows_sync({
        json: true,
        code: "dmc.msig",
        scope: pname,
        table: "approvals",
        limit: 50
    });
    var table = new AsciiTable();
    table.setHeading('', 'Name')
    a.rows.forEach(function(d, index) {
        table.addRow(index + 1, d.proposal_name)
    });

    while (true) {
        console.notice(table.toString());
        var index = Number(console.readLine("choice NO:\n"));
        if (index && a.rows[index - 1]) break;
    }
    showApprovalsInfo(pname, index - 1);
}

function showApprovalsInfo(pname, pindex) {
    var a = dmc.rpc.get_table_rows_sync({
        json: true,
        code: "dmc.msig",
        scope: pname,
        table: "approvals",
        limit: 50
    });
    var appr = [];
    var appinfo = a.rows[pindex];
    if (!appinfo) {
        showApprovals(pname)
        return;
    }
    var table = new AsciiTable(appinfo.proposal_name);

    table.setHeading('', 'actor', 'permission', 'approve');
    console.log(appinfo)
    appinfo.provided_approvals.map(function(d) {
        d.provided = true;
        return d;
    });
    var arr = appinfo.provided_approvals.concat(appinfo.requested_approvals);
    arr.forEach(function(dd, index) {
        table.addRow(index + 1, dd.actor, dd.permission, dd.provided ? "ok" : "");
    });
    console.notice(table.toString());
    choice();
}